function [filters,order,nlsp,fs] = adfread(filename)

%[filters,order,nlsp,fs] = adfread(filename)
%
% Reads a matrix of FIR shelving filters for modified high-frequency Ambisonics 
% decoding from a '.adf'-file.
%
% Parameters:
%   'filename'..... filename, excluding file ending
%
%   'filters'...... a (filterorder+1)-by-(Ambisonics order+1) matrix, as
%                   returned from 'ambimodgains'
%   'order'........ Ambisonics order
%   'nlsp'......... number of loudspeakers
%   'fs'........... sampling frequency [Hz]
%
% - Brd Stfringsdal 2006 -

% add file type
filename = [filename '.adf'];

% open
fid = fopen(filename,'r');
if fid == -1
  error('Can not open file');
end

% read header
filetype = fread(fid,3,'uint8=>char');
if ~strcmp(filetype.','ADF')
  error('Wrong filetype')
end
fs = fread(fid,1,'int');
order = fread(fid,1,'int');
filterlength = fread(fid,1,'int');
nlsp = fread(fid,1,'int');

% content
filters = fread(fid,inf,'float');
% reshape
filters = reshape(filters,filterlength,order+1);

fclose(fid);
