function createadf(type,scaling,fc,order,orderlim,nlsp,fs,filterlength,filename)

%function createadf(type,scaling,fc,order,orderlim,nlsp,fs,filterlength,filename)
%
% Calculates FIR shelving filters for modified high-frequency Ambisonics decoding.
% The filters will be exported to a binary file which can be loaded into ambidec.
%
% Parameters:
%   'type'......... 'maxre' or 'inphase'.
%                   can be a two element cell array, related to 'fc'.
%   'scaling'...... 'pressure' or 'energy' are valid values
%   'fc'........... center frequency for shelving filters.
%                   can be a two element vector
%   'order'........ Ambisonics order
%   'orderlim'..... force basic decoding up to 'orderlim'
%   'nlsp'......... number of loudspeakers (assumes regular 2D layout)
%   'fs'........... sampling frequency [Hz]
%   'filterlength'. length of each FIR filter, will be set equal to a power of two
%                   [samples]
%   'filename'..... descriptive name, excluding file ending
%
%
% References:
% J. Daniel, Reprsentation de Champs Acoustiques, Application  la Transmission et  la 
%   Reproduction de Scnes Sonores Complexes dans un Contexte Multimdia, Ph.D. Thesis, 
%   University of Paris 6, Paris, France, 2000, Section 3.3.2. 
%   (http://gyronymo.free.fr/audio3D/download_Thesis_PwPt.html)
%
% - Brd Stfringsdal 2006 -

% fix filterlength
filterlength = 2^nextpow2(filterlength);

% parameters
transwidth = 1; % [octaves]
nf = filterlength;
fmax = fs/2;

% create filters
[throwone throwtwo filters] = ambimodgains(order,orderlim,fc,transwidth,nf,fmax,type,scaling,nlsp,filterlength);

% write to file
adfwrite(filters,nlsp,fs,filename);