/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "ALSAMIDI.hpp"
#include <stdexcept>

void* ALSAMIDI::midi_thread(void* arg)
{
	snd_seq_event_t* e;
	ALSAMIDI* c;

	c = static_cast<ALSAMIDI*>(arg);
	for (;;) {
		// Block while waiting for an event.
		snd_seq_event_input(c->_seq, &e);
		// Check if we're connected to an object
		if (!c->_process) continue;
		switch (e->type) {
		case SND_SEQ_EVENT_NOTEON:
			// Relevant: e->data.note.[channel|note|velocity]
			break;
		case SND_SEQ_EVENT_CONTROLLER:
			c->_process->MIDIMessage(MIDIReceiver::Controller,
			    e->data.control.channel, e->data.control.param,
			    e->data.control.value);
			break;
		case SND_SEQ_EVENT_PORT_SUBSCRIBED:
			// MIDI port connected
			break;
		case SND_SEQ_EVENT_PORT_UNSUBSCRIBED:
			// MIDI port disconnected
			break;
		default:
			// Something else
			break;
		}
	}
	return NULL;
}

ALSAMIDI::ALSAMIDI(char* name) : _seq(0), _process(0)
{
	if (snd_seq_open(&_seq, "default", SND_SEQ_OPEN_INPUT, 0) < 0)
		throw std::runtime_error("Can't open ALSA sequencer");
	snd_seq_set_client_name(_seq, name);
	if (snd_seq_create_simple_port(_seq, name, SND_SEQ_PORT_CAP_WRITE |
	    SND_SEQ_PORT_CAP_SUBS_WRITE, SND_SEQ_PORT_TYPE_APPLICATION) < 0)
		throw std::runtime_error("Can't create ALSA sequencer port");
	if (pthread_create(&_threadid, NULL, &ALSAMIDI::midi_thread, this) != 0)
		throw std::runtime_error("Can't create ALSA sequencer thread");
}

void ALSAMIDI::SetProcess(MIDIReceiver& o)
{
	_process = &o;
}

ALSAMIDI::~ALSAMIDI()
{
	if (_seq) {
		pthread_cancel(_threadid);
		pthread_join(_threadid, NULL);
		snd_seq_close(_seq);
	}
}

