/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _alsamidi_h
#define _alsamidi_h

#include <alsa/asoundlib.h>
#include <pthread.h>
#include "MIDIReceiver.hpp"

/** 
 * \brief Class for receiving MIDI messages via the ALSA MIDI sequencer
 * interface.
 * 
 * Is capable of sending MIDI messages to any class derived from the
 * MIDIReceiver class.
 */
class ALSAMIDI {
private:
	snd_seq_t* _seq;
	pthread_t _threadid;
	MIDIReceiver* _process;
	static void* midi_thread(void* arg);
public:
	/**
	 * Class constructor. Throws exceptions of type std::runtime_error when an
	 * error occurs.
	 * @param name under what name to register the MIDI port.
	 */
	ALSAMIDI(char* name);
	/**
	 * Sets the class that should be notified of MIDI messages.
	 * @param o class to be notified.
	 */
	void SetProcess(MIDIReceiver& o);
	/**
	 * Class destructor.
	 */
	~ALSAMIDI();
};

#endif

