/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _ambisonicsdecoder_h
#define _ambisonicsdecoder_h

#include "Ambisonics.hpp"
#include "AudioProcessor.hpp"
#include "Defs.hpp"
#include "DSPTools.hpp"
#include "FFTFilter.hpp"
#include "Matrix.hpp"
#include "PositionVector.hpp"
#include "SIMDMath.hpp"

class AmbisonicsDecoder : public AudioProcessor {
	SIMDMathPtr _math;
	Matrix _matrix;
	PositionVector _pos;
	FloatVector _gainramp;
	std::vector<PartitionedFilter> _filters;
	// Stuff for filtering.
	FFTConvolver _convolver;
	InputBuffer _firinput;
	OutputBuffer _firoutput;
	std::vector<float*> _firoutputptrs;
	unsigned _order;
	bool _is3d;
	float _gain, _matrixgain;
	bool _input_filters_enabled;
	Gainer _outputgainer;
	Ambisonics _ambi;
	void InitMatrix();
public:
	AmbisonicsDecoder();
	void SetNumSpeakers(size_t num);
	void SetIs3D(bool is3d);
	void SetOrder(unsigned order);
	void SetGain(float gain);
	void UpdateSpeaker(size_t spk);
	void UpdateAllSpeakers();
	void SetFilters(std::vector<FloatVector>& filters);
	bool InputFiltersEnabled() const;
	void SetNormalization(Ambisonics::Normalization norm);
	void SetRegularisationParam(double regparm);
	void SetAzimuth(size_t spk, double azimuth);
	void SetElevation(size_t spk, double elevation);
	virtual void SetSampleRate(unsigned sr);
	virtual void SetBufferSize(unsigned size);
	virtual void Process(float** inputs, float** outputs, unsigned num);
	virtual size_t GetNumInputs() const
	{
		return _matrix.size2();
	}
	virtual size_t GetNumOutputs() const
	{
		return _matrix.size1();
	}
	virtual ~AmbisonicsDecoder()
	{
	}
};

#endif

