/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _ambisonicsdecodergui_h
#define _ambisonicsdecodergui_h

#include <gtkmm/actiongroup.h>
#include <gtkmm/adjustment.h>
#include <gtkmm/button.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/scale.h>
#include <gtkmm/statusbar.h>
#include <gtkmm/toggleaction.h>
#include <gtkmm/uimanager.h>
#include <gtkmm/window.h>
#include "AmbisonicsDecoderModel.hpp"
#include "DecoderSettingsWindow.hpp"
#include "JACKInterface.hpp"
#include "MeterWindow.hpp"
#include "SpeakerWindow.hpp"
#include "View.hpp"
#include "Widgets.hpp"

class AmbisonicsDecoderGUI : public Gtk::Window, public View {
private:
	static const Glib::ustring _menu_info;
	Glib::RefPtr<Gtk::ActionGroup> _actiongroup;
	Glib::RefPtr<Gtk::UIManager> _uimanager;
	Gtk::Adjustment _adj_numspeakers, _adj_order;
	Gtk::SpinButton _btn_numspeakers, _btn_order;
	Gtk::CheckButton _btn_3d;
	GainSlider _sld_volume;
	Gtk::Statusbar _statusbar;
	Glib::RefPtr<Gtk::ToggleAction> _act_speakerlayout, _act_settings,
	                                _act_meters;
	AmbisonicsDecoderModel& _decoder;
	SpeakerWindow _wnd_speakers;
	DecoderSettingsWindow _wnd_settings;
	MeterWindow _wnd_meters;
	JACKInterface& _snd;
	void on_numspeakers_changed();
	void on_order_changed();
	void on_3d_clicked();
	void on_volume_changed();
	void on_quit();
	void on_load_filters();
	void on_save();
	void on_open();
	void on_view_speakers();
	bool on_hide_speakers(GdkEventAny* e);
	void on_view_settings();
	bool on_hide_settings(GdkEventAny* e);
	void on_view_meters();
	bool on_hide_meters(GdkEventAny* e);
	void on_route_inputs();
	void on_route_outputs();
	void on_view_aboutbox();
public:
	AmbisonicsDecoderGUI(AmbisonicsDecoderModel& dec, JACKInterface& snd);
	virtual void Notify(int what);
};

#endif
