/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _ambisonicsencodergui_h
#define _ambisonicsencodergui_h

#include <gtkmm/actiongroup.h>
#include <gtkmm/adjustment.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/menu.h>
#include <gtkmm/scale.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/toggleaction.h>
#include <gtkmm/uimanager.h>
#include <gtkmm/window.h>
#include "AmbisonicsEncoderModel.hpp"
#include "EncoderSettingsWindow.hpp"
#include "JACKInterface.hpp"
#include "MeterWindow.hpp"
#include "SourceWindow.hpp"
#include "View.hpp"

class AmbisonicsEncoderGUI : public Gtk::Window, public View {
private:
	static const Glib::ustring _menu_info;
	Glib::RefPtr<Gtk::ActionGroup> _actiongroup;
	Glib::RefPtr<Gtk::UIManager> _uimanager;
	Gtk::Adjustment _adj_numsources, _adj_order, _adj_lspdistance;
	Gtk::SpinButton _btn_numsources, _btn_order;
	Gtk::CheckButton _btn_3d;
	Gtk::HScale _sld_lspdistance;
	Glib::RefPtr<Gtk::ToggleAction> _act_sources, _act_settings, _act_meters;
	AmbisonicsEncoderModel& _encoder;
	SourceWindow _wnd_sources;
	EncoderSettingsWindow _wnd_settings;
	MeterWindow _wnd_meters;
	JACKInterface& _snd;
	void on_numsources_changed();
	void on_order_changed();
	void on_3d_clicked();
	void on_lsp_distance_changed();
	void on_open();
	void on_save();
	void on_quit();
	void on_view_sources();
	bool on_hide_sources(GdkEventAny* e);
	void on_view_settings();
	bool on_hide_settings(GdkEventAny* e);
	void on_view_meters();
	bool on_hide_meters(GdkEventAny* e);
	void on_route_inputs();
	void on_route_outputs();
	void on_view_aboutbox();
public:
	AmbisonicsEncoderGUI(AmbisonicsEncoderModel& enc, JACKInterface& snd);
	virtual void Notify(int what);
};

#endif

