/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _butterworthfilter_h
#define _butterworthfilter_h

#include <cmath>
#include <vector>
#include "IIRFilter.hpp"

class ButterworthFilter : public IIRFilter<float> {
public:
	enum Type { Lowpass, Highpass };
protected:
	unsigned _order;
private:
	static std::vector<double> _designcoefs;
	static unsigned _precalcorder;
	Type _type;
	double _cutoff_mult;
	void CalcCoefTable(unsigned order);
public:
	ButterworthFilter(unsigned sr, Type type = Lowpass, unsigned maxorder = 0)
	: _order(0)
	{
		SetSampleRate(sr);
		SetType(type);
		CalcCoefTable(maxorder);
	}

	void SetType(Type type)
	{
		_type = type;
	}

	void SetSampleRate(unsigned sr)
	{
		_cutoff_mult = M_PI/double(sr);
	}

	void SetOrder(unsigned order);
	void SetCutoff(double cutoff);
};

#endif

