/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "DecoderSettingsWindow.hpp"
#include <gtkmm/notebook.h>
#include <gtkmm/table.h>

DecoderSettingsWindow::DecoderSettingsWindow(AmbisonicsDecoderModel& dec)
	: _btn_regparm(0., 5), _dec(dec)
{
	set_title("Decoder settings");
	set_border_width(5);
	set_default_size(300, 200);

	Gtk::Table* table1 = Gtk::manage(new Gtk::Table(2, 3));
	Gtk::Notebook* tabs = Gtk::manage(new Gtk::Notebook());
	// Build a GUI consisting of a NoteBook with one table in each tab.
	table1->set_border_width(16);
	table1->set_col_spacing(0, 20);
	table1->set_col_spacing(1, 10);
	// First page labels
	Gtk::Label* label1 = Gtk::manage(new Gtk::Label("Input normalization:"));
	label1->set_alignment(Gtk::ALIGN_RIGHT);
	Gtk::Label* label2 = Gtk::manage(new Gtk::Label("Regularization parameter:"));
	label2->set_alignment(Gtk::ALIGN_RIGHT);
	Gtk::Label* label3 = Gtk::manage(new Gtk::Label("(0 = off)"));
	label3->set_alignment(Gtk::ALIGN_LEFT);

	_btn_regparm.set_range(0., 0.2);
	_btn_regparm.set_increments(0.001, 0.01);
	// Populate first page
	table1->attach(*label1, 0, 1, 0, 1, Gtk::FILL);
	table1->attach(_combo_norm, 1, 2, 0, 1, Gtk::FILL | Gtk::EXPAND, Gtk::EXPAND);
	table1->attach(*label2, 0, 1, 1, 2, Gtk::FILL);
	table1->attach(_btn_regparm, 1, 2, 1, 2, Gtk::FILL | Gtk::EXPAND, Gtk::EXPAND);
	table1->attach(*label3, 2, 3, 1, 2, Gtk::FILL);

	// Connect signal handlers
	_combo_norm.signal_changed().connect(sigc::mem_fun(
		*this, &DecoderSettingsWindow::on_normalization_changed));
	_btn_regparm.signal_value_changed().connect(sigc::mem_fun(
		*this, &DecoderSettingsWindow::on_regparm_changed));
	tabs->append_page(*table1, "General");
	add(*tabs);
	show_all_children();
}

void DecoderSettingsWindow::Notify(int what)
{
	switch (what) {
	case AmbisonicsDecoderModel::ChangedNormalization:
		_combo_norm.set_norm(_dec.Normalization());
		break;
	case AmbisonicsDecoderModel::ChangedRegularisationParam:
		_btn_regparm.set_value(_dec.RegularisationParam());
		break;
	case AmbisonicsDecoderModel::ChangedAll:
		_combo_norm.set_norm(_dec.Normalization());
		_btn_regparm.set_value(_dec.RegularisationParam());
		break;
	}
}

void DecoderSettingsWindow::on_normalization_changed()
{
	_dec.Normalization(_combo_norm.get_norm());
}

void DecoderSettingsWindow::on_regparm_changed()
{
	_dec.RegularisationParam(_btn_regparm.get_value());
}

