/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _fftw_h
#define _fftw_h

#include <fftw3.h>
#include <string>

// TODO fix interface

/**
 * \brief Simple class wrapping the FFTW library for performing real data FFTs.
 */
class FFTW {
private:
	const long _length;
	fftwf_plan _fft, _ifft;
public:
	typedef float flt_t;
	/**
	 * See if there is an FFTW wisdom file available, and if not, create it.
	 * Creating it might take some time.
	 * @param filename name of wisdom file to be loaded/created. Will be assumed
	 * to be located in users home dir.
	 */
	static void HandleWisdom(std::string filename);
	/**
	 * Class constructor. Will create an FFT and IFFT plan of specified length.
	 * @param length FFT length. Should be power of two.
	 */
	FFTW(const long length);
	/**
	 * Performs an FFT.
	 * @param f output array, containing frequency domain data.
	 * @param x input array, containing time domain data.
	 */
	void do_fft(flt_t* f, flt_t* x);
	/**
	 * Performs an IFFT.
	 * @param f input array, containing frequency domain data.
	 * @param x output array, containing time domain data.
	 */
	void do_ifft(flt_t* f, flt_t* x);
	/**
	 * Rescales data according to normalisation factor 1/N.
	 * @param x data to be normalised.
	 */
	void rescale(flt_t* x) const;
	/**
	 * Class destructor.
	 */
	~FFTW();
};

#endif
