/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _levelmeter_h
#define _levelmeter_h

#include "config.h"
#include <gdkmm/colormap.h>
#include <glibmm/timer.h>
#include <gtkmm/drawingarea.h>

// Graphic and set_level() method taken from Meterbridge by Steve Harris.
// http://plugin.org.uk/meterbridge/

#ifdef HAVE_CAIRO
#define METER_FGCOLOUR 1., 0., 0.
#define METER_PEAKCOLOUR 1., 1., 1.
#endif

class LevelMeter : public Gtk::DrawingArea
{
	Glib::RefPtr<Gdk::Window> _window;
#ifdef HAVE_CAIRO
	Cairo::RefPtr<Cairo::Context> _cr;
	static Cairo::RefPtr<Cairo::ImageSurface> _bitmap;
#else
	Gdk::Color _fgcolour, _peakcolour;
	Glib::RefPtr<Gdk::GC> _gc;
	static Glib::RefPtr<Gdk::Pixbuf> _bitmap;
#endif
	double _level, _peak;
	double _peak_time;
	Glib::Timer _peak_timer;
protected:
	virtual void on_realize();
	virtual bool on_expose_event(GdkEventExpose* e);
	virtual void on_size_request(Gtk::Requisition* req);
	virtual void on_size_allocate(Gtk::Allocation& alloc);
public:
	LevelMeter();
	void set_level(double l);
};

#endif

