/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _midireceiver_h
#define _midireceiver_h

/**
 * \brief Class for receiving MIDI messages.
 *
 * See the ALSAMIDI class for corresponding interface class.
 */
class MIDIReceiver {
public:
	/**
	 * Type describing what kind of message has been received.
	 */
	enum MessageType { NoteOn, NoteOff, Controller };
	/**
	 * Method that will be called when a MIDI message is ready.
	 * @param msg Type of message. See MessageType for explanation.
	 * @param chan MIDI channel.
	 * @param num interpretation depends on messagetype. For note messages, it's
	 * note numbers, for controller changes, it's control number.
	 * @param val Value of message in question.
	 */
	virtual void MIDIMessage(MessageType msg, char chan, 
	                         char num, char val) = 0;
	/**
	 * Class destructor.
	 */
	virtual ~MIDIReceiver()
	{
	}
};
#endif

