/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _meterwindow_h 
#define _meterwindow_h

#include <gtkmm/box.h>
#include <gtkmm/label.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/window.h>
#include <vector>
#include "AudioProcessor.hpp"
#include "LevelMeter.hpp"
#include "View.hpp"

class MeterWindow : public Gtk::Window, public View
{
	class MeterStrip : public Gtk::VBox
	{
		Gtk::Label _label;
		LevelMeter _meter;
	public:
		MeterStrip(Glib::ustring name) : _label(name)
		{
			_label.set_alignment(Gtk::ALIGN_CENTER);
			pack_start(_meter, Gtk::PACK_SHRINK);
			pack_start(_label, Gtk::PACK_SHRINK);
			show_all_children();
		}
		void queue_draw_level(float level)
		{
			_meter.set_level(level);
			// These are the coords for the part of the meter we need to redraw,
			// no need to waste time redrawing the meter bitmap
			_meter.queue_draw_area(7, 5, 9, 200);
		}
	};

	Gtk::HBox _ibox, _obox;
	std::vector<MeterStrip*> _istrips, _ostrips;
	AudioInterface& _snd;
	size_t _num_imeters_disp, _num_ometers_disp;
	bool redraw_timer();
	void set_num_meters(std::vector<MeterStrip*>& strips, size_t& num_disp,
	                    Gtk::HBox& box, size_t num);
public:
	MeterWindow(AudioInterface& snd);
	virtual void Notify(int what);
	void on_show();
	void on_hide();
	virtual ~MeterWindow()
	{
	}
};

#endif

