/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _nfcbassboost_h
#define _nfcbassboost_h

#include "ButterworthFilter.hpp"

class NFCHighpass : public ButterworthFilter {
	double _conv_mult;
public:
	NFCHighpass(unsigned sr, unsigned maxorder = 0, double c = 344.)
	: ButterworthFilter(sr, Highpass, maxorder)
	{
		_conv_mult = c/(2.*M_PI);
	}

	void SetDistance(double dist, bool is3d)
	{
		const double kr_2d[] = { 1.405, 2.355, 3.417, 4.429, 5.393,
		                         6.404, 7.409, 8.357, 9.347, 10.345
		                       };
		const double kr_3d[] = { 1.669, 2.836, 3.919, 4.981, 6.031,
		                         7.074, 8.110, 9.132, 10.147, 11.171
		                       };
		const double kr = is3d ? kr_3d[_order - 1] : kr_2d[_order - 1];
		double cutoff = _conv_mult*kr/dist;

		SetCutoff(std::max(cutoff, 15.));
	}
};
#endif

