/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "ParameterFilter.hpp"
#include <cmath>
ParameterFilter::ParameterFilter(double scale, double init)
{
	SetScale(scale);
	// Initalise filter history so that parameters can start out from a
	// point of choice, not always from 0.
	SetPos(init);
}

void ParameterFilter::SetPos(double pos)
{
	_ah = pos/(1. - _ac);
	_bh = pos/(1. - _bc);
	_ch = pos/(1. - _cc);
}		

double ParameterFilter::GetPos() const
{
	return _ag*_ah + _bg*_bh + _cg*_ch;
}

void ParameterFilter::SetScale(double scale)
{
	// 0 < Smoothness < 1. High is better, but may cause precision problems
	const double smoothness = 0.999;
	const double oldpos = GetPos();
	const double a = 1. - 2.4/scale;
	const double b = smoothness;
	_ac = a;
	_bc = a*b;
	_cc = a*b*b;
	const double loga = log(a);
	const double logab = log(a*b);
	const double logabb = log(a*b*b);
	const double mastergain = loga*logab*logabb/
	                          (logabb*(2*loga - logab) - logab*loga);
	_ag = mastergain;
	_bg = mastergain*
	      (logabb*(loga - logab)/((logabb - logab)*logab) - loga/logab);
	_cg = mastergain*((logab - loga)/(logabb - logab));
	SetPos(oldpos);
}
void ParameterFilter::Process(float* y, float val, unsigned num)
{
	for (unsigned i = 0; i < num; ++i) {
		_ah = _ac*_ah + val;
		_bh = _bc*_bh + val;
		_ch = _cc*_ch + val;
		/* Combine filters in parallel */
		y[i] = _ag*_ah + _bg*_bh + _cg*_ch;
	}
}

