/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _parameterfilter_h
#define _parameterfilter_h

/**
 * \brief Filter for smoothing out parameter changes.
 *
 * Constructed from three one pole filters in parallel.
 * Adapted from http://www.musicdsp.org/archive.php?classid=3#28.
 * Original code by Olli Niemitalo.
 */
class ParameterFilter {
private:
	double _ac, _bc, _cc; // Filter coefficients
	double _ag, _bg, _cg; // Gain coefficients
	double _ah, _bh, _ch; // Filter history
public:
	/**
	 * Class constructor.
	 * @param scale number of samples from start of parameter change to midpoint
	 * of smoothed curve.
	 * @param init initial value of parameter in question. State of the internal
	 * filter coefficients will be set to reflect this value in the output.
	 */
	ParameterFilter(double scale, double init = 0.);
	/**
	 * Make the filter jump immediately to a given value with no smoothing
	 * period in between.
	 * @param what value to set the filter output to.
	 */
	void SetPos(double pos);
	/**
	 * Queries the interpolator for the current interpolated position.
	 * @return The interpolated position.
	 */
	double GetPos() const;
	/**
	 * Sets the number of samples until midpoint of interpolated parameter curve
	 * occurs.
	 * @param number of samples.
	 */	
	void SetScale(double scale);
	/**
	 * Performs actual parameter smoothing.
	 * @param y array in which to put interpolated values.
	 * @param current parameter value.
	 * @param num number of values to create.
	 */
	void Process(float* y, float val, unsigned num);
};

#endif

