/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _positionvector_h
#define _positionvector_h

#include <cmath>
#include <vector>

/**
 * \brief A single polar coordinate position.
 *
 * Class has no accessor methods for simplicity.
 */
struct Position {
	double radius, azimuth, elevation;
	Position() : radius(1.), azimuth(0.), elevation(0.) { }
};

typedef std::vector<Position> PositionVector_t;

/**
 * \brief Vector of polar coordinate positions.
 *
 * Class is very small, and uses no accessor methods for simplicity.
 * @see Position
 */
class PositionVector : public PositionVector_t {
public:		
	/**
	 * Makes a circle.
	 * @param a azimuth of the first point on the circle, measured in clockwise
	 * direction from straight forward.
	 * @param radius radius of circle. Defaults to 1.
	 * @param scale angle unit scale. Defaults to radians (2*pi).
	 */
	void MakeCircle(double a, double radius = 1., double scale = 2.*M_PI)
	{
		PositionVector::iterator speaker = begin();
		double delta = scale/size();
		
		for (size_t i = 0; i < size(); ++i, ++speaker) {
			speaker->radius = radius;
			speaker->azimuth = fmod(a + i*delta, scale);
			speaker->elevation = 0.;
		}
	}
};

#endif

