/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _sourcewindow_h 
#define _sourcewindow_h

#include <gtkmm/checkbutton.h>
#include <gtkmm/label.h>
#include <gtkmm/notebook.h>
#include <gtkmm/scale.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/table.h>
#include <gtkmm/window.h>
#include <vector>
#include "AmbisonicsEncoderModel.hpp"
#include "PositionMap.hpp"
#include "Widgets.hpp"

typedef std::vector<Gtk::SpinButton*> SpinButtonVector;
typedef std::vector<Gtk::CheckButton*> CheckButtonVector;
typedef std::vector<Gtk::HScale*> HScaleVector;

class SourceWindow : public Gtk::Window, public GtkView {
private:
	Gtk::Notebook _tabs;
	// Slider view widgets
	HScaleVector _azimuthsliders;
	HScaleVector _radiussliders;
	HScaleVector _elevationsliders;
	HScaleVector _gainsliders;
	CheckButtonVector _pwavebuttons;
	Gtk::Table _table;

	// Graphical view widgets
	PositionMap _posmap;
	Gtk::SpinButton _btn_source, _btn_azimuth, _btn_radius, _btn_elevation;
	Gtk::CheckButton _btn_pwave;
	Gtk::Label _posxyz_label;
	GainSlider _sld_gain;
	Gtk::HScale _sld_scale;
	
	AmbisonicsEncoderModel& _enc;
	void update_xyz_label(size_t num);
	void on_azimuth_changed(size_t i);
	void on_elevation_changed(size_t i);
	void on_radius_changed(size_t i);
	void on_pwave_toggled(size_t i);
	void on_gain_changed(size_t i);
	void on_map_pos_active(size_t num);
	void on_map_pos_changed(size_t num);
	void on_active_source_changed();
	void on_scale_changed();
	void on_switch_tab(GtkNotebookPage* page, guint num);
	void CreateRow(size_t pos);
	void MakeRows(size_t num);
public:
	SourceWindow(AmbisonicsEncoderModel& enc);
	void UpdateControls();
	void notify();
};

#endif

