/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef _speakerwindow_h 
#define _speakerwindow_h

#include <gtkmm/adjustment.h>
#include <gtkmm/notebook.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/table.h>
#include <gtkmm/window.h>
#include <vector>
#include "AmbisonicsDecoderModel.hpp"
#include "PositionMap.hpp"
#include "PositionVector.hpp"
#include "View.hpp"

typedef std::vector<Gtk::SpinButton*> SpinButtonVector;

class SpeakerWindow : public Gtk::Window, public View {
private:
	Gtk::Notebook _tabs;

	// Control view widgets
	SpinButtonVector _azimuthwidgets;
	SpinButtonVector _elevationwidgets;
	Gtk::ScrolledWindow _scrolledwindow;
	Gtk::RadioButton _btn_custom, _btn_circle;
	Gtk::Adjustment _adj_azimuth;
	Gtk::SpinButton _btn_azimuth;
	Gtk::Table _table;

	// Graphical view widgets
	PositionMap _posmap;

	AmbisonicsDecoderModel& _dec;
	void CreateRow(size_t pos);
	void CreateRows();
	void on_button_close();
	void on_azimuth_changed(size_t i);
	void on_elev_changed(size_t i);
	void on_firstazimuth_changed();
	void on_layout_clicked();
	void on_map_pos_changed(size_t num);
	void on_switch_tab(GtkNotebookPage* page, guint num);
public:
	SpeakerWindow(AmbisonicsDecoderModel& dec);
	void UpdateControls();
	virtual void Notify(int what);
	virtual ~SpeakerWindow()
	{
	}
};

#endif

