/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <gtkmm/main.h>
#include <iostream>
#include "AmbisonicsDecoderGUI.hpp"
#include "AmbisonicsDecoderModel.hpp"
#include "JACKInterface.hpp"

char clientname[] = "ambidec";

int main(int argc, char** argv)
{
	std::cout << "Initialising FFTW, this might take a while.\n";
	FFTW::HandleWisdom(".ambitools_wisdom");

	try {    
		JACKInterface sound(clientname);
		AmbisonicsDecoderModel decodermodel(sound);
		Gtk::Main kit(argc, argv);
		AmbisonicsDecoderGUI gui(decodermodel, sound);
	
		kit.run(gui);
		
    	// We're back. Disable JACK driver so it doesn't attempt to call the
		// processor class after destruction.
		sound.DisableProcessing();
		return 0;
	}
	catch (const Glib::Error& e) {
		std::cerr << "GLib exception: " << e.what() << std::endl;
		return 1;
	}
	catch (const std::exception& e) {
		std::cerr << "Exception: " << e.what() << std::endl;
		return 1;
	}
	catch (...) {
		std::cerr << "Unknown exception occured.\n";
		return 1;
	}
}

