/* 
 * This file is part of ambitools.
 * Copyright (C) 2006-2007 Centre for Quantifiable Quality of Service in
 * Communication Systems, Norwegian University of Science and Technology.
 *
 * ambitools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ambitools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ambitools; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <boost/scoped_ptr.hpp>
#include <exception>
#include <gtkmm/main.h>
#include <iostream>
#include "ALSAMIDI.hpp"
#include "AmbisonicsEncoderGUI.hpp"
#include "AmbisonicsEncoderModel.hpp"
#include "JACKInterface.hpp"

char clientname[] = "ambienc";

int main(int argc, char** argv)
{
	try {
		JACKInterface sound(clientname);
		AmbisonicsEncoderModel encodermodel(sound);
		Gtk::Main kit(argc, argv);
		AmbisonicsEncoderGUI gui(encodermodel, sound);
		boost::scoped_ptr<ALSAMIDI> midi;

		// Try to init MIDI without it crashing everything on fail. We don't
		// require it to work.
		try {
			midi.reset(new ALSAMIDI(clientname));
		}
		catch (std::runtime_error& e) {
			std::cout << "ALSA MIDI error: " << e.what() << std::endl;
			std::cout << "ALSA MIDI interface disabled.\n";
		}
		
		if (midi)
			midi->SetProcess(encodermodel);
		kit.run(gui);
		// We're back. Disable JACK driver so it doesn't attempt to call the
		// processor class after destruction.
		sound.DisableProcessing();
	    return 0;
	}
	catch (const Glib::Error& e) {
		std::cerr << "GLib exception: " << e.what() << std::endl;
		return 1;
	}
	catch (const std::exception& e) {
		std::cerr << "Exception: " << e.what() << std::endl;
		return 1;
	}
	catch (...) {
		std::cerr << "Unknown exception occured.\n";
		return 1;
	}
}
